# == Resource type git::config
#
# Full description of class git here.
#
# === Parameters
#
# [*title*]
#   Can be in the form section.key.
#
# [*ensure*]
#   Manage the presence of the configuration key.
#
# [*dir*]
#   Specify a git repository to which apply the configuration.
#
# [*key*]
#   Configuration key to manage. Default guessed from title.
#
# [*section*]
#   Configuration section for the key to manage. Default guessed from title.
#
# [*user*]
#   Apply the configuration for the user. Default 'root'.
#
# [*value*]
#   Value of the configuration. This string must escape single and double
#   quotes with a backslash.
#
# [*zone*]
#   Which configuration file to modify.
#   If dir is set, zone is not used.
#   Default : 'system' if no user is specified, else 'global'.
#
#
# === Examples
#
#  git::config { 'color.ui':
#    value => true,
#  }
#
#  git::config { 'color.ui':
#    value => true,
#  }
#
# === Authors
#
# Remi Sauvat <remi.sauvat+puppet@inetprocess.com>
#
# === Copyright
#
# Copyright 2014 iNet Process
#
define git::config (
  $ensure   = 'present',
  $dir      = undef,
  $key      = regsubst($title, '^([^\.]+)\.([^\.]+)$','\2'),
  $section  = regsubst($title, '^([^\.]+)\.([^\.]+)$','\1'),
  $user     = undef,
  $value    = '',
  $zone     = undef,
) {

  include ::git::params

  $_zone_tpl = inline_template("<%=
  unless @dir
    if @zone
      '--' << @zone
    else
      if @user
        '--global'
      else
        '--system'
      end
    end
  end
  -%>")

  $_user = $user ? {
    undef   => 'root',
    default => $user,
  }
  $type = "${section}.${key}"
  # Login as the specified user to set the HOME variable
  $su_prefix = [$::git::params::su, '-l', '-c']
  $su_suffix = [$_user]
  $git_prefix = [$::git::params::bin, 'config', $_zone_tpl]

  $escaped_value = regexpescape($value)
  $get_value = ['--get', $type, "^${escaped_value}$"]
  $get = ['--get', $type]
  $set = ['--replace-all', $type, $value]
  $unset = ['--unset-all', $type]

  if $ensure == 'present' {
    $command = shell_join($su_prefix + [shell_join($git_prefix +  $set), $_user])
    $unless = shell_join($su_prefix + [shell_join($git_prefix + $get_value) , $_user])
    $onlyif = undef
  } else {
    $command = shell_join($su_prefix + [shell_join($git_prefix + $unset), $_user])
    $unless = undef
    $onlyif = shell_join($su_prefix + [shell_join($git_prefix + $get), $_user])
  }

  exec{"git_config_${title}" :
    command => $command,
    path    => ['/usr/bin', '/bin', '/usr/local/bin'],
    onlyif  => $onlyif,
    unless  => $unless,
    cwd     => $dir,
    require => Package[$::git::params::package_name],
  }
}

